
' Using Coordinate Spaces and Transformations

' This example demonstrates the following tasks using SetWorldTransform():

'    * Drawing graphics with predefined units.
'    * Centering graphics in the application's client area.
'    * Scaling graphics output to half its original size.
'    * Translating graphics output 3/4 of an inch to the right.
'    * Rotating graphics 30 degrees.
'    * Shearing graphics output along the x-axis.
'    * Reflecting graphics output about an imaginary horizontal axis 
'		   drawn through its midpoint.

VERSION	"0.0001"

	IMPORT  "xst_s.lib"
	IMPORT  "xsx_s.lib"
'	IMPORT  "xio_s.lib"
'	IMPORT	"xst"   		' standard library	: required by most programs
'	IMPORT  "xsx"				' extended std library
'	IMPORT	"xio"				' console io library
	IMPORT	"gdi32"     ' gdi32.dll
	IMPORT  "user32"    ' user32.dll
	IMPORT  "kernel32"  ' kernel32.dll
	IMPORT  "comctl32"	' comctl32.dll			: common controls library

DECLARE FUNCTION Entry            ()
DECLARE FUNCTION WndProc          (hwnd, msg, wParam, lParam)
DECLARE FUNCTION InitGui          ()
DECLARE FUNCTION CreateWindows    ()
DECLARE FUNCTION NewWindow        (titleBar$, style, x, y, w, h, exStyle, addrWndProc, icon$, menu$)
DECLARE FUNCTION MessageLoop      ()
DECLARE FUNCTION TransformAndDraw (iTransform, hWnd)

$$NORMAL 		= 100
$$SCALE     = 101
$$TRANSLATE = 102
$$ROTATE		= 103
$$SHEAR			= 104
$$REFLECT   = 105

FUNCTION Entry ()

	InitGui ()												' initialize program and libraries
	IF CreateWindows () THEN QUIT (0)	' create windows and other child controls
	MessageLoop ()										' the main message loop

END FUNCTION

FUNCTION WndProc (hWnd, msg, wParam, lParam)

	PAINTSTRUCT ps
	RECT rect
	STATIC iTransform

	SELECT CASE msg

		CASE $$WM_COMMAND :
			iTransform = LOWORD (wParam)
			InvalidateRect (hWnd, NULL, 1)
			
		CASE $$WM_DESTROY :
			PostQuitMessage(0)

		CASE $$WM_PAINT :
			hdc = BeginPaint (hWnd, &ps)
			TransformAndDraw (iTransform, hWnd)
			EndPaint (hWnd, &ps)

		CASE ELSE :
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)

	END SELECT

END FUNCTION

FUNCTION InitGui ()

	SHARED hInst

	hInst = GetModuleHandleA (0) 	' get current instance handle
	IFZ hInst THEN QUIT (0)
	InitCommonControls ()					' initialize comctl32.dll library
	
END FUNCTION

FUNCTION CreateWindows ()

	' create main window
	titleBar$ 	= "Using Coordinate Spaces and Transforms"
	style 			= $$WS_OVERLAPPEDWINDOW
	w 					= 500
	h 					= 500
	exStyle			= 0
	addrWndProc = &WndProc()
	icon$ 			= "scrabble"
	menu$ 			= "menubar"
	#winMain = NewWindow (titleBar$, style, x, y, w, h, exStyle, addrWndProc, icon$, menu$)
	IFZ #winMain THEN RETURN ($$TRUE)

	XstCenterWindow (#winMain)							' center window position
	ShowWindow (#winMain, $$SW_SHOWNORMAL)	' show window
	UpdateWindow (#winMain)

END FUNCTION

FUNCTION NewWindow (titleBar$, style, x, y, w, h, exStyle, addrWndProc, icon$, menu$)

	SHARED hInst
	WNDCLASS wc
	
	className$ = titleBar$
	IFZ className$ THEN className$ = "Main Program"
	IF INSTR(className$, " ") THEN
		XstReplace (@className$, " ", "_", 0)
	END IF
	
	IFZ icon$ THEN icon$ = "scrabble"
	
	wc.style           = $$CS_HREDRAW | $$CS_VREDRAW | $$CS_OWNDC
	wc.lpfnWndProc     = addrWndProc
	wc.cbClsExtra      = 0
	wc.cbWndExtra      = 0
	wc.hInstance       = hInst
	wc.hIcon           = LoadIconA (hInst, &icon$)
	wc.hCursor         = LoadCursorA (0, $$IDC_ARROW)
	wc.hbrBackground   = GetStockObject ($$WHITE_BRUSH)
	wc.lpszMenuName    = &menu$
	wc.lpszClassName   = &className$

	IFZ RegisterClassA (&wc) THEN RETURN
	RETURN CreateWindowExA (exStyle, &className$, &titleBar$, style, x, y, w, h, 0, 0, hInst, 0)

END FUNCTION

FUNCTION MessageLoop ()

	MSG msg

' main message loop

	IF LIBRARY(0) THEN RETURN								' main program executes message loop

	DO																			' the message loop
		ret = GetMessageA (&msg, NULL, 0, 0)	' retrieve next message from queue

		SELECT CASE ret
			CASE  0 : RETURN msg.wParam					' WM_QUIT message
			CASE -1 : RETURN $$TRUE							' error
			CASE ELSE:
				hwnd = GetActiveWindow ()
				IF (!IsWindow (hwnd)) || (!IsDialogMessageA (hwnd, &msg)) THEN		' send only non-dialog messages
  				TranslateMessage (&msg)					' translate virtual-key messages into character messages
  				DispatchMessageA (&msg)					' send message to window callback function WndProc()
				END IF
		END SELECT
	LOOP

END FUNCTION


FUNCTION TransformAndDraw (iTransform, hWnd)

	XFORM xForm 
	RECT rect 

	' Retrieve a DC handle for the application's window. 

	hDC = GetDC(hWnd) 

	' Set the mapping mode to LOENGLISH. This moves the 
	' client area origin from the upper left corner of the 
	' window to the lower left corner (this also reorients 
	' the y-axis so that drawing operations occur in a true 
	' Cartesian space). It guarantees portability so that 
	' the object drawn retains its dimensions on any display. 

	SetGraphicsMode (hDC, $$GM_ADVANCED)
	SetMapMode (hDC, $$MM_LOENGLISH) 

	' Set the appropriate world transformation (based on the 
	' user's menu selection). 

	SELECT CASE (iTransform) 

		CASE $$SCALE:        ' Scale to 1/2 of the original size. 
				xForm.eM11 =  0.5 
				xForm.eM12 =  0.0 
				xForm.eM21 =  0.0 
				xForm.eM22 =  0.5 
				xForm.eDx  =  0.0 
				xForm.eDy  =  0.0 
				SetWorldTransform (hDC, &xForm) 

		CASE $$TRANSLATE:   ' Translate right by 3/4 inch. 
				xForm.eM11 =  1.0 
				xForm.eM12 =  0.0 
				xForm.eM21 =  0.0 
				xForm.eM22 =  1.0 
				xForm.eDx  =  75.0 
				xForm.eDy  =  0.0 
				SetWorldTransform (hDC, &xForm) 

		CASE $$ROTATE:      ' Rotate 30 degrees counterclockwise. 
				xForm.eM11 =  0.8660 
				xForm.eM12 =  0.5000 
				xForm.eM21 =  -0.5000 
				xForm.eM22 =  0.8660 
				xForm.eDx  =  0.0 
				xForm.eDy  =  0.0 
				SetWorldTransform (hDC, &xForm) 

		CASE $$SHEAR:       ' Shear along the x-axis with a 
												' proportionality constant of 1.0. 
				xForm.eM11 =  1.0 
				xForm.eM12 =  1.0 
				xForm.eM21 =  0.0 
				xForm.eM22 =  1.0 
				xForm.eDx  =  0.0 
				xForm.eDy  =  0.0 
				SetWorldTransform (hDC, &xForm) 

		CASE $$REFLECT:     ' Reflect about a horizontal axis. 
				xForm.eM11 =  1.0 
				xForm.eM12 =  0.0 
				xForm.eM21 =  0.0 
				xForm.eM22 =  -1.0 
				xForm.eDx  =  0.0 
				xForm.eDy  =  0.0 
				SetWorldTransform (hDC, &xForm) 

		CASE $$NORMAL:      ' Set the unity transformation. 
				xForm.eM11 =  1.0 
				xForm.eM12 =  0.0 
				xForm.eM21 =  0.0 
				xForm.eM22 =  1.0 
				xForm.eDx  =  0.0 
				xForm.eDy  =  0.0 
				SetWorldTransform (hDC, &xForm) 
	END SELECT

	' Find the midpoint of the client area. 
	GetClientRect (hWnd, &rect) 
	DPtoLP (hDC, &rect, 2) 

	' Select a hollow brush. 
	SelectObject (hDC, GetStockObject($$HOLLOW_BRUSH)) 

	' Draw the exterior circle. 
	Ellipse (hDC, (rect.right / 2 - 100), (rect.bottom / 2 + 100),	(rect.right / 2 + 100), (rect.bottom / 2 - 100)) 

	' Draw the interior circle. 
	Ellipse (hDC, (rect.right / 2 -94), (rect.bottom / 2 + 94), (rect.right / 2 + 94), (rect.bottom / 2 - 94)) 

	' Draw the key. 
	Rectangle (hDC, (rect.right / 2 - 13), (rect.bottom / 2 + 113), (rect.right / 2 + 13), (rect.bottom / 2 + 50)) 
	Rectangle (hDC, (rect.right / 2 - 13), (rect.bottom / 2 + 96),	(rect.right / 2 + 13), (rect.bottom / 2 + 50)) 

	' Draw the horizontal lines. 
	MoveToEx (hDC, (rect.right/2 - 150), (rect.bottom / 2 + 0), NULL) 
	LineTo (hDC, (rect.right / 2 - 16), (rect.bottom / 2 + 0)) 

	MoveToEx (hDC, (rect.right / 2 - 13), (rect.bottom / 2 + 0), NULL) 
	LineTo (hDC, (rect.right / 2 + 13), (rect.bottom / 2 + 0)) 

	MoveToEx (hDC, (rect.right / 2 + 16), (rect.bottom / 2 + 0), NULL) 
	LineTo (hDC, (rect.right / 2 + 150), (rect.bottom / 2 + 0)) 

	' Draw the vertical lines. 
	MoveToEx (hDC, (rect.right/2 + 0), (rect.bottom / 2 - 150), NULL) 
	LineTo (hDC, (rect.right / 2 + 0), (rect.bottom / 2 - 16)) 

	MoveToEx (hDC, (rect.right / 2 + 0), (rect.bottom / 2 - 13), NULL) 
	LineTo (hDC, (rect.right / 2 + 0), (rect.bottom / 2 + 13)) 

	MoveToEx (hDC, (rect.right / 2 + 0), (rect.bottom / 2 + 16), NULL) 
	LineTo (hDC, (rect.right / 2 + 0), (rect.bottom / 2 + 150)) 

	ReleaseDC (hWnd, hDC) 

END FUNCTION
END PROGRAM
